﻿B4A=true
Group=Default Group
ModulesStructureVersion=1
Type=StaticCode
Version=8.5
@EndOfDesignText@
#Region Copyright
'  Project: atHome
'  Copyright (C) 2018 Robert W.B. Linn, Pinneberg, Germany.
'
'  Constants: Code Module JSONParser to parse Domoticz server response GET requests
'
'  Version: 20180925
'
'  This project is free software; you can redistribute it And/Or
'  modify it under the terms of the GNU Lesser General Public
'  License As published by the Free Software Foundation; either
'  version 2 of the License, Or (at your option) any later version.
'
'  This project Is distributed in the hope that it will be useful,
'  but WITHOUT ANY WARRANTY; without even the implied warranty of
'  MERCHANTABILITY Or FITNESS For A PARTICULAR PURPOSE. See the GNU
'  Lesser General Public License For more details.
'
'  You should have received a copy of the GNU Lesser General Public
'  License along with this project; If Not, write To the
'  Free Software Foundation, Inc., 59 Temple Place - Suite 330,
'  Boston, MA 02111-1307, USA.
#End Region

#Region Notes
'Subs in this code module will be accessible from all modules.
' Online JSON parser:https://www.b4x.com:51041/json/index.html
#End Region

'These global variables will be declared once when the application starts.
'These variables can be accessed from all modules.
Sub Process_Globals
	' List for each of the TABS holding the devices idx used to populate the tabs
	Public ListDevicesWeather As List
	Public ListDevicesLights As List
	Public ListDevicesInformation As List
	' JSON Response globals to access keys
	Public ResponseString As String
	Private ResponseRoot As Map
	Private ResponseResult As List
End Sub

' Build a list of idx for selected device according types & subtypes.
' TODO: Find a way to flexible assigning device types&subtypes to one of the tabs. No idea yet how to do best.
Public Sub GetDevices(Response As String) As Boolean
	' Dim Result As Boolean = True
	Dim Parser As JSONParser
	Dim ParserDeviceMap As JSONParser
	Dim DeviceMapRoot As Map
	Dim WeatherDevices As List
	Dim LightsDevices As List
	Dim InformationDevices As List

	' Flag if the device has been added. if not, the device is unknown
	Dim DeviceAdded As Boolean
	' Check if the JSONString has content
	If Response.Trim.Length = 0 Then
		Return False
	End If
	
	' Init the global lists
	ListDevicesWeather.Initialize
	ListDevicesLights.Initialize
	ListDevicesInformation.Initialize

	' Init the the JSON Response String Parser
	Parser.Initialize(Response)
	' Get the root of the JSON structure
	ResponseRoot = Parser.NextObject

	' Init the device mapping parser
	ParserDeviceMap.Initialize(File.ReadString(File.DirAssets, "devicemapping.json"))
	' Get the root of the JSON structure
	DeviceMapRoot = ParserDeviceMap.NextObject
	WeatherDevices = DeviceMapRoot.Get("weather")
	LightsDevices = DeviceMapRoot.Get("lights")
	InformationDevices = DeviceMapRoot.Get("information")

	' Get generic keys
	' Dim app_version As String = ResponseRoot.Get("app_version")
	' Dim ServerTime As String = ResponseRoot.Get("ServerTime")

	' Get the result flag OK and status
	' Dim title As String = ResponseRoot.Get("title")
	' Dim status As String = ResponseRoot.Get("status")

	' List of devices from the key result"
	ResponseResult = ResponseRoot.Get("result")
	
	For Each colresult As Map In ResponseResult
		' Set the flag to check if the device has been found
		DeviceAdded = False

		' Set properties
		Dim DevIdx As String = colresult.Get("idx")
		Dim DevType As String = colresult.Get("Type")
		Dim DevSubType As String = colresult.Get("SubType")
		' Assign the Type & SubType to one of the lists = Tabs
		' Compare the Type And SubTypes from the DevType Or SubType Value

		'WEATHER
		For Each m As Map In WeatherDevices
			If (DevType = m.Get("type")) And (DevSubType = m.Get("subtype")) Then
				ListDevicesWeather.Add(DevIdx)
				DeviceAdded = True				
			End If			
		Next

		' LIGHTS
		For Each m As Map In LightsDevices
			If (DevType = m.Get("type")) And (DevSubType = m.Get("subtype")) Then
				ListDevicesLights.Add(DevIdx)
				DeviceAdded = True
			End If
		Next

		' INFORMATION
		For Each m As Map In InformationDevices
			If (DevType = m.Get("type")) And (DevSubType = m.Get("subtype")) Then
				ListDevicesInformation.Add(DevIdx)
				DeviceAdded = True
			End If
		Next

		' Handle device not found. Add to the information tab.
		If Not(DeviceAdded) Then
			ListDevicesInformation.Add(DevIdx)
		End If

	Next
	Return True
End Sub

' Get the key value for a generic JSON object - not part of the property result (array)
' IMPORTANT: key must match exact!
' Response - Holding key:value pairs
' Key - Key (property) of the device
' Returns the value as a string or text "unknown" if the device or key was not found.
Public Sub GetGenericKey(Response As String, Key As String) As String
	Dim Result As String = "unknown"
	Dim Parser As JSONParser
	Dim RootObject As Map

'	Log("JSONGetGenericKey:" & Key)
	' Parse the JSON Response String
	Parser.Initialize(Response)

	' Get the root of the JSON structure
	RootObject = Parser.NextObject

	' Get generic keys, i.e
	' ResponseRoot.Get("app_version")
	' ResponseRoot.Get("ServerTime")
	If (Response.Trim.Length > 0) Then
'		Log("Found:" & Key & "=" & RootObject.Get(Key))
		Return RootObject.Get(Key)
	End If
	Return Result
End Sub

' Get the value for a device key.
' IMPORTANT: key must match exact!
' Returns the value as a string or text "unknown" if the device or key was not found.
Public Sub GetDeviceKey(Response As String, Idx As Int, Key As String) As String
	Dim Result As String = "unknown"
	Dim Parser As JSONParser
	Dim RootObject As Map
	Dim ResultArray As List

'	Log("JSONGetDeviceKey:" & Idx & "," & Key)
	' Parse the JSON Response String
	Parser.Initialize(Response)
	' Get the root of the JSON structure
	RootObject = Parser.NextObject
	' Check if response content
	If (Response.Trim.Length = 0) Then
		Return Result
	End If

	' Get the array of devices from the results key
	ResultArray = RootObject.Get("result")

	If ResultArray.IsInitialized Then
		' Get the key for a device idx
		' List of devices from the key result"
		For Each Device As Map In ResultArray
			Dim DevIdx As Int = Device.Get("idx")
			If DevIdx = Idx Then
'			Log("Found:" & Idx & "," & Key & "=" & colresult.Get(Key))
				Return Device.Get(Key)
			End If
		Next		
	End If

	Return Result
End Sub

' Get the device properties as mao from the result key. The result key holds an array of devices.
' Returns an initialised map if device found else an uninitialised map
Public Sub GetDeviceProperties(Response As String, Idx As Int) As Map
	Dim Result As Map
	Dim Parser As JSONParser
	Dim RootObject As Map
	Dim ResultArray As List

'	Log("JSONGetDeviceData:" & Idx)

	' Parse the JSON Response String
	Parser.Initialize(Response)
	' Get the root of the JSON structure
	RootObject = Parser.NextObject
	' Check if response content
	If (Response.Trim.Length = 0) Then
		Return Result
	End If

	' Get the array of devices from the results key
	ResultArray = RootObject.Get("result")

	If ResultArray.IsInitialized Then
		' Get the key for a device idx
		' List of devices from the key result"
		For Each Device As Map In ResultArray
			Dim DevIdx As Int = Device.Get("idx")
			If DevIdx = Idx Then
				' Return initialised map with device data
				Return Device
			End If
		Next
	End If

	' Return uninitialised map
  	Return Result
End Sub

